﻿using System.Collections.Generic;
using System.Linq;
using System.Text;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.Code;
using OpcLabs.DotProlog.Errors;
using OpcLabs.DotProlog.Execution;
using OpcLabs.DotProlog.Execution.Extensions;
using OpcLabs.DotProlog.InputOutput;
using OpcLabs.DotProlog.Terms;
using OpcLabs.DotProlog.Test;

namespace DotPrologTest.Execution
{
    [TestClass]
    // ReSharper disable once InconsistentNaming
    public class Catch_3_Test : ExecutionTest
    {
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Test_7_8_9_3_Errors_a()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("catch(G, c, r). ");
                TestError(engine, goal, ErrorTerms.InstantiationError);
            }
        }

        [TestMethod]
        public void Test_7_8_9_3_Errors_b()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("catch(1, c, r). ");
                TestError(engine, goal, ErrorTerms.TypeError(ValidTypes.Callable, new PInteger(1)));
            }
        }

        [TestMethod]
        public void Test_7_8_9_4_Examples_1()
        {
            using (IEngine engine = new Engine())
            {
                Consult(engine.Program);

                Term goal = Parse("catch(foo(5), test(Y), true). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedY = new PInteger(10);
                var expected = new VariableDictionary {{variableNameDictionary["Y"], expectedY}};
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_9_4_Examples_2()
        {
            using (IEngine engine = new Engine())
            {
                Consult(engine.Program);

                Term goal = Parse("catch(bar(3), Z, true). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedZ = new PInteger(3);
                var expected = new VariableDictionary {{variableNameDictionary["Z"], expectedZ}};
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_9_4_Examples_3()
        {
            using (IEngine engine = new Engine())
            {
                Consult(engine.Program);

                Term goal = Parse("catch(true, _, 3). ", out IDictionary<string, Variable> variableNameDictionary);
                Test(engine, goal, Success(), "Expected success.");
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_9_4_Examples_4()    // TODO: actually this is not correct - need to throw system_error!
        {
            using (IEngine engine = new Engine())
            {
                Consult(engine.Program);

                Term goal = Parse("catch(true, C, write(demoen)), throw(bla). ", out IDictionary<string, Variable> variableNameDictionary);
                try
                {
                    // ReSharper disable once ReturnValueOfPureMethodIsNotUsed
                    engine.Call(goal).First();
                }
                catch (PrologException prologException)
                {
                    Term ball = prologException.Ball;
                    Assert.IsNotNull(ball);
                    Assert.IsNotNull(TermUtilities.UnifyWithoutOccursCheck(Atom.Add("bla"), ball));
                    return;
                }
                Assert.Fail();
            }
        }

        [TestMethod]
        public void Test_7_8_9_4_Examples_5()
        {
            using (IEngine engine = new Engine())
            {
                Consult(engine.Program);

                Term goal = Parse("catch(car(X), Y, true). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedY = new PInteger(1);
                var expected = new VariableDictionary { { variableNameDictionary["Y"], expectedY } };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_9_4_Examples_6()
        {
            using (IEngine engine = new Engine())
            {
                Consult(engine.Program);

                Term goal = Parse("catch(number_chars(X, ['l', 'a', 'O']), error(syntax_error(_), _), fail). ", out IDictionary<string, Variable> variableNameDictionary);
                Test(engine, goal, Failure(), "Expected failure.");
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_9_4_Examples_7()
        {
            var stringBuilder = new StringBuilder();
            using (IEngine engine = new Engine { Environment = { CurrentOutputStream = TextStream.Create(stringBuilder) } })
            {
                Consult(engine.Program);

                Term goal = Parse("catch(g, C, write(h1)). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedC = Atom.Add("c");
                var expected = new VariableDictionary { { variableNameDictionary["C"], expectedC } };
                Test(engine, goal, Success(expected));
                Assert.AreEqual("h1", stringBuilder.ToString());
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_9_4_Examples_8()
        {
            using (IEngine engine = new Engine())
            {
                Consult(engine.Program);

                Term goal = Parse("catch(coo(X), Y, true). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedY = new NTerm("error") { ErrorTerms.InstantiationError, "[]"};
                var expected = new VariableDictionary { { variableNameDictionary["Y"], expectedY } };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        // IMPROVE: have them all from the spec

        // ReSharper restore InconsistentNaming



        // ReSharper disable once SuggestBaseTypeForParameter
        static private void Consult([NotNull] IProgram program)
        {
            IDatabase database = program.Database;

            database.Assert(ParseClause("foo(X) :- Y is X * 2, throw(test(Y))."));

            database.Assert(ParseClause("bar(X) :- X = Y, throw(Y)."));

            database.Assert(ParseClause("coo(X) :- throw(X)."));

            database.Assert(ParseClause("car(X) :- X = 1, throw(X)."));

            database.Assert(ParseClause("g :- catch(p, B, write(h2)), coo(c)."));

            database.Assert(ParseClause("p."));
            database.Assert(ParseClause("p :- throw(b)."));
        }
    }
}
