﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.Execution;
using OpcLabs.DotProlog.Terms;
using OpcLabs.DotProlog.Test;

namespace DotPrologTest.Execution
{
    [TestClass]
    // ReSharper disable once InconsistentNaming
    public class IfThenElse_2_Test : ExecutionTest
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Test_7_8_8_4_Examples_1()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse("';'('->'(true, true), fail). "), Success(), "Expected success.");
            }
        }

        [TestMethod]
        public void Test_7_8_8_4_Examples_2()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse("';'('->'(fail, true), true). "), Success(), "Expected success.");
            }
        }

        [TestMethod]
        public void Test_7_8_8_4_Examples_3()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse("';'('->'(true, fail), fail). "), Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_7_8_8_4_Examples_4()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse("';'('->'(fail, true), fail). "), Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_7_8_8_4_Examples_5()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("';'('->'(true, X=1), X=2). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedX = new PInteger(1);
                var expected = new VariableDictionary { { variableNameDictionary["X"], expectedX } };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_8_4_Examples_6()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("';'('->'(fail, X = 1), X = 2). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedX = new PInteger(2);
                var expected = new VariableDictionary { { variableNameDictionary["X"], expectedX } };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_8_4_Examples_7()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("';'('->'(true, ';'(X=1, X=2)), true). ", out IDictionary<string, Variable> variableNameDictionary);
                var expected1 = new VariableDictionary { { variableNameDictionary["X"], new PInteger(1) } };
                var expected2 = new VariableDictionary { { variableNameDictionary["X"], new PInteger(2) } };
                Test(engine, goal, Success(expected1, expected2));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_8_4_Examples_8()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("';'('->'(';'(X=1, X=2), true), true). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedX = new PInteger(1);
                var expected = new VariableDictionary { { variableNameDictionary["X"], expectedX } };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_7_8_8_4_Examples_9()
        {
            using (IEngine engine = new Engine())
            {
                // Cor.1
                Test(engine, Parse("';'(('->'(!,fail), true), true). "), Success(), "Expected success.");
            }
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
