﻿using DotPrologTest.Execution;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.Errors;
using OpcLabs.DotProlog.Terms;
using OpcLabs.DotProlog.Test;

namespace DotPrologTest.Predicates.AtomicTermProcessing
{
    [TestClass]
    // ReSharper disable InconsistentNaming
    public class AtomChars_2_Test : SemiDeterministicPredicateTest
    // ReSharper restore InconsistentNaming
    {
        public AtomChars_2_Test() : base("atom_chars", 2)
        {
        }

        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Test1()
        {
            Atom atom = Atom.Add("atom");
            var list = new Variable();
            Test(atom, list, new VariableDictionary{{list, new NList{"a","t","o","m"}}});
        }

        [TestMethod]
        public void Test2()
        {
            var atom = new Variable();
            Term list = new NList{"a","t","o","m"};
            Test(atom, list, new VariableDictionary {{atom, Atom.Add("atom")}});
        }

        [TestMethod]
        public void Test_8_16_4_3_Errors_a_1()
        {
            var atom = new Variable();
            Term list = new NList(new Variable()) {"a"};
            TestError(atom, list, ErrorTerms.InstantiationError);
        }

        [TestMethod]
        public void Test_8_16_4_3_Errors_a_2()
        {
            var atom = new Variable();
            Term list = new NList { "a", new Variable() };
            TestError(atom, list, ErrorTerms.InstantiationError);
        }

        [TestMethod]
        public void Test_8_16_4_3_Errors_b()
        {
            var atom = new PInteger(1);
            Term list = new NList { "a", "t", "o", "m" };
            TestError(atom, list, ErrorTerms.TypeError(ValidTypes.Atom, atom));
        }

        [TestMethod]
        public void Test_8_16_4_3_Errors_c()
        {
            var atom = new Variable();
            var list = new PInteger(1);
            TestError(atom, list, ErrorTerms.TypeError(ValidTypes.List, list));
        }

        [TestMethod]
        public void Test_8_16_4_3_Errors_d_1()
        {
            var atom = new Variable();
            Term list = new NList { "a", 1 };
            TestError(atom, list, ErrorTerms.TypeError(ValidTypes.Character, new PInteger(1)));
        }

        [TestMethod]
        public void Test_8_16_4_3_Errors_d_2()
        {
            var atom = new Variable();
            Term list = new NList { "a", "" };
            TestError(atom, list, ErrorTerms.TypeError(ValidTypes.Character, Atom.Add("")));
        }

        [TestMethod]
        public void Test_8_16_4_3_Errors_d_3()
        {
            var atom = new Variable();
            Term list = new NList { "a", "to" };
            TestError(atom, list, ErrorTerms.TypeError(ValidTypes.Character, Atom.Add("to")));
        }

        [TestMethod]
        public void Test_8_16_4_4_Examples_1()
        {
            Atom atom = Atom.Add("");
            var list = new Variable();
            Test(atom, list, new VariableDictionary {{ list, new NList()}});
        }

        [TestMethod]
        public void Test_8_16_4_4_Examples_2()
        {
            Atom atom = Atom.Add("[]");
            var L = new Variable();
            Test(atom, L, new VariableDictionary {{ L, new NList{"[","]"}}});
        }

        [TestMethod]
        public void Test_8_16_4_4_Examples_3()
        {
            Atom atom = Atom.Add("'");
            var L = new Variable();
            Test(atom, L, new VariableDictionary { { L, new NList { "'" } } });
        }

        [TestMethod]
        public void Test_8_16_4_4_Examples_4()
        {
            Atom atom = Atom.Add("ant");
            var L = new Variable();
            Test(atom, L, new VariableDictionary { { L, new NList { "a","n","t" } } });
        }

        [TestMethod]
        public void Test_8_16_4_4_Examples_5()
        {
            var Str = new Variable();
            Term list = new NList {"s","o","p"};
            Test(Str, list, new VariableDictionary { { Str, Atom.Add("sop")} });
        }

        [TestMethod]
        public void Test_8_16_4_4_Examples_6()
        {
            Atom atom = Atom.Add("North");
            var X = new Variable();
            Term list = new NList(X) { "N" };
            Test(atom, list, new VariableDictionary { { X, new NList {"o","r","t","h"} } });
        }

        [TestMethod]
        public void Test_8_16_4_4_Examples_7()
        {
            Atom atom = Atom.Add("soap");
            Term list = new NList { "s", "o", "p" };
            Test(atom, list, false);
        }

        [TestMethod]
        public void Test_8_16_4_4_Examples_8()
        {
            var X = new Variable();
            var Y = new Variable();
            TestError(X, Y, ErrorTerms.InstantiationError);
        }

        // ReSharper restore InconsistentNaming
    }
}
