﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.Errors;
using OpcLabs.DotProlog.Execution;
using OpcLabs.DotProlog.Terms;
using OpcLabs.DotProlog.Test;

namespace DotPrologTest.Predicates.TermUnification
{
    [TestClass]
    // ReSharper disable InconsistentNaming
    public class _NotPrologUnifiable_2_Test : SemiDeterministicPredicateTest
    // ReSharper restore InconsistentNaming
    {
        public _NotPrologUnifiable_2_Test()
            : base(@"\=", 2)
        {
        }

        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Test_8_2_3_4_Examples_1()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse(@"'\\='(1, 1). "), Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_2()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse(@"\=(X, 1). ");
                Test(engine, goal, Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_3()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse(@"'\\='(X, Y). ");
                Test(engine, goal, Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_4()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse(@"\=(_, _). ");
                Test(engine, goal, Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_5()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse(@"\=(f(X, def), f(def, Y)). ");
                Test(engine, goal, Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_6()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse(@"'\\='(1, 2). "), Success(), "Expected success.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_7()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse(@"\=(1, 1.0). "), Success(), "Expected success.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_8()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"'\\='( g(X), " +
                        @"       f(f(X)) ). "),
                    Success(),
                    "Expected success.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_9()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"\=( f(X,  1), " +
                        @"    f(a(X) ) ). "),
                    Success(),
                    "Expected success.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_10()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"'\\='( f(X,    Y,    X   ), " +
                        @"       f(a(X), a(Y), Y, 2) ). "),
                    Success(),
                    "Expected success.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_11()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"\=( X, " +
                        @"    a(X) ). "),
                    Failure(),
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_12()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"'\\='( f(X,    1), " +
                        @"       f(a(X), 2) ). "),
                    Success(),
                    "Expected success.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_13()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"'\\='( f(1, X,    1), " +
                        @"       f(2, a(X), 2) ). "),
                    Success(),
                    "Expected success.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_14()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"\=( f(2, X), " +
                        @"    f(2, a(X)) ). "),
                    Failure(),
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_3_4_Examples_15()
        {
            using (IEngine engine = new Engine())
            {
                TestError(
                    engine,
                    Parse(
                        @"'\\='( f(X,    Y,    X, 1), " +
                        @"       f(a(X), a(Y), Y, 2) ). "),
                    ErrorTerms.ResourceError("execution_stack"));
            }
        }

        // ReSharper restore InconsistentNaming
    }
}
