﻿using System.Collections.Generic;
using System.Diagnostics;
using JetBrains.Annotations;
using OpcLabs.DotProlog.Code;
using OpcLabs.DotProlog.Debugging;
using OpcLabs.DotProlog.Execution;
using OpcLabs.DotProlog.Terms;

namespace OpcLabs.DotProlog.Test
{
    public class PredicateTest : ExecutionTest
    {
        #region Public Constructors

        public PredicateTest([NotNull] Functor functor)
        {
            Functor = functor;
        }

        public PredicateTest([NotNull] string name, int arity) : this(new Functor(name, arity))
        {
        }

        #endregion



        #region Static Public Methods

        #endregion



        #region Public Properties

        [NotNull]
        public Functor Functor { get; }

        #endregion



        #region Public Methods

        public void TestPredicate([NotNull] IList<Term> arguments, [NotNull] IEnumerable<VariableDictionary> expected)
        {
            TestPredicate(arguments, expected, "");
        }

        public void TestPredicate([NotNull] IList<Term> arguments, [NotNull] IEnumerable<VariableDictionary> expected, long maximumResultCount)
        {
            TestPredicate(arguments, expected, maximumResultCount, "");
        }

        public void TestPredicate(
            [NotNull] IList<Term> arguments,
            [NotNull] IEnumerable<VariableDictionary> expected,
            [NotNull] string message)
        {
            TestPredicate(arguments, expected, -1, message);
        }

        public void TestPredicate(
            [NotNull] IList<Term> arguments, 
            [NotNull] IEnumerable<VariableDictionary> expected,
            long maximumResultCount,
            [NotNull] string message)
        {
            Debug.Assert(arguments.Count == Functor.Arity);
            Term goal = new NTerm(Functor.Atom, arguments);
            Debug.Assert(!(goal is null));
            
            IDebugger debugger = CreateDebugger();
            IProgram program = CreateProgram();

            using (IEngine engine = new Engine(program, debugger))
                Test(engine, goal, expected, maximumResultCount, message);
        }

        public void TestPredicate([NotNull] IEnumerable<VariableDictionary> expected)
        {
            TestPredicate(new List<Term>(), expected, "");
        }

        public void TestPredicate([NotNull] IEnumerable<VariableDictionary> expected, long maximumResultCount)
        {
            TestPredicate(new List<Term>(), expected, maximumResultCount, "");
        }

        public void TestPredicate([NotNull] IEnumerable<VariableDictionary> expected, [NotNull] string message)
        {
            TestPredicate(new List<Term>(), expected, message);
        }

        public void TestPredicate([NotNull] Term arg, [NotNull] IEnumerable<VariableDictionary> expected)
        {
            TestPredicate(new List<Term> { arg }, expected, "");
        }

        public void TestPredicate([NotNull] Term arg, [NotNull] IEnumerable<VariableDictionary> expected, [NotNull] string message)
        {
            TestPredicate(new List<Term> {arg}, expected, message);
        }

        public void TestPredicate(
            [NotNull] Term arg1,
            [NotNull] Term arg2,
            [NotNull] IEnumerable<VariableDictionary> expected)
        {
            TestPredicate(new List<Term> { arg1, arg2 }, expected, "");
        }

        public void TestPredicate(
            [NotNull] Term arg1, 
            [NotNull] Term arg2, 
            [NotNull] IEnumerable<VariableDictionary> expected, 
            [NotNull] string message)
        {
            TestPredicate(new List<Term> { arg1, arg2 }, expected, message);
        }

        #endregion



        #region Protected Methods

        [NotNull]
        protected virtual IDebugger CreateDebugger() => TracingDebugger.Default;

        [NotNull]
        protected virtual IProgram CreateProgram() => new Program();

        #endregion
    }
}
