﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib;
using OpcLabs.BaseLib.Browsing;
using OpcLabs.BaseLib.Navigation;
using OpcLabs.BaseLib.Navigation.Parsing;
using OpcLabs.BaseLib.Parsing;

namespace OpcLabs.BaseLibTest.Browsing
{
    
    
    /// <summary>
    ///This is a test class for BrowsePathTest and is intended
    ///to contain all BrowsePathTest Unit Tests
    ///</summary>
    public partial class BrowsePathTest
    {
        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        ///A test for BrowsePath Constructor
        ///</summary>
        [TestMethod]
        public void BrowsePathConstructorTest()
        {
            IEnumerable<string> browseNames = new [] {"ElementA", "ElementB"};
            var target = new BrowsePath(browseNames);
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB" }, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for BrowsePath Constructor
        ///</summary>
        [TestMethod]
        public void BrowsePathConstructorTest1()
        {
            const string value = "/ElementA/ElementB";
            var target = new BrowsePath(value);
            CollectionAssert.AreEqual(new [] {"ElementA", "ElementB"}, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for BrowsePath Constructor
        ///</summary>
        [TestMethod]
        public void BrowsePathConstructorTest2()
        {
            var browsePath = new BrowsePath(new[] { "ElementA", "ElementB" });
            var target = new BrowsePath(browsePath);
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB" }, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for BrowsePath Constructor
        ///</summary>
        [TestMethod]
        public void BrowsePathConstructorTest3()
        {
            var basePath = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string value = "ElementC/ElementD";
            var target = new BrowsePath(basePath, value);
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB", "ElementC", "ElementD" }, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for BrowsePath Constructor
        ///</summary>
        [TestMethod]
        public void BrowsePathConstructorTest4()
        {
            var target = new BrowsePath();
            CollectionAssert.AreEqual(new string[0], target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for BrowsePath Constructor
        ///</summary>
        [TestMethod]
        public void BrowsePathConstructorTest5()
        {
            var basePath = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string value = "/ElementC/ElementD";
            var target = new BrowsePath(basePath, value);
            CollectionAssert.AreEqual(new[] { "ElementC", "ElementD" }, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for AppendBrowseName
        ///</summary>
        [TestMethod]
        public void AppendBrowseNameTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string browseName = "ElementC";
            target.AppendBrowseName(browseName);
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB", "ElementC" }, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for AppendRelative
        ///</summary>
        [TestMethod]
        public void AppendRelativeTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string more = "ElementC/ElementD";
            target.AppendRelative(more);
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB", "ElementC", "ElementD" }, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for AppendRelative
        ///</summary>
        [TestMethod]
        public void AppendRelativeTest1()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            IEnumerable<string> more = new[] {"ElementC", "ElementD"};
            target.AppendRelative(more);
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB", "ElementC", "ElementD" }, target.BrowseNames.ToList());
        }

        // IMPROVE: put the below back
        ///// <summary>
        /////A test for CheckFormatError
        /////</summary>
        //[TestMethod]
        //public void CheckFormatErrorTest()
        //{
        //    try
        //    {
        //        const BrowsePathFormatErrorCode formatError = BrowsePathFormatErrorCode.RelativeWithNoBase;
        //        BrowsePathParserUtilities.CheckFormatError(formatError);
        //    }
        //    catch (BrowsePathFormatException browsePathFormatException)
        //    {
        //        Assert.AreEqual(BrowsePathFormatErrorCode.RelativeWithNoBase, browsePathFormatException.FormatErrorCode);
        //        return;
        //    }
        //    Assert.Fail();
        //}

        ///// <summary>
        /////A test for CheckFormatError
        /////</summary>
        //[TestMethod]
        //public void CheckFormatErrorTest1()
        //{
        //    const BrowsePathFormatErrorCode formatError = BrowsePathFormatErrorCode.None;
        //    BrowsePath.CheckFormatError(formatError);
        //}

        /// <summary>
        ///A test for Combine
        ///</summary>
        [TestMethod]
        public void CombineTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string more = "ElementC/ElementD";
            target.Combine(more);
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB", "ElementC", "ElementD" }, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for Combine
        ///</summary>
        [TestMethod]
        public void CombineTest1()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string more = "/ElementC/ElementD";
            target.Combine(more);
            CollectionAssert.AreEqual(new[] { "ElementC", "ElementD" }, target.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for CommonPrefix
        ///</summary>
        [TestMethod]
        public void CommonPrefixTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB", "ElementC1", "ElementD1" });
            var path2 = new BrowsePath(new[] { "ElementA", "ElementB", "ElementC2"});
            IEnumerable<string> expected = new[] { "ElementA", "ElementB"};
            IEnumerable<string> actual = target.CommonPrefix(path2);
            CollectionAssert.AreEqual(expected.ToList(), actual.ToList());
        }

        /// <summary>
        ///A test for CommonPrefixLength
        ///</summary>
        [TestMethod]
        public void CommonPrefixLengthTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB", "ElementC1", "ElementD1" });
            var path2 = new BrowsePath(new[] { "ElementA", "ElementB", "ElementC2" });
            const int expected = 2;
            int actual = target.CommonPrefixLength(path2);
            Assert.AreEqual(expected, actual);
        }

        /*TODO:
        /// <summary>
        ///A test for Equals
        ///</summary>
        [TestMethod]
        public void EqualsTest()
        {
            BrowsePath browsePath = null; // TODO: Initialize to an appropriate value
            BrowsePath target = new BrowsePath(browsePath); // TODO: Initialize to an appropriate value
            BrowsePath other = null; // TODO: Initialize to an appropriate value
            bool expected = false; // TODO: Initialize to an appropriate value
            bool actual;
            actual = target.Equals(other);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }

        /// <summary>
        ///A test for Equals
        ///</summary>
        [TestMethod]
        public void EqualsTest1()
        {
            BrowsePath browsePath = null; // TODO: Initialize to an appropriate value
            BrowsePath target = new BrowsePath(browsePath); // TODO: Initialize to an appropriate value
            object obj = null; // TODO: Initialize to an appropriate value
            bool expected = false; // TODO: Initialize to an appropriate value
            bool actual;
            actual = target.Equals(obj);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }
         * */

        /// <summary>
        ///A test for EscapeBrowseName
        ///</summary>
        [TestMethod]
        public void EscapeBrowseNameTest()
        {
            string actual = BrowsePath.EscapeBrowseName("Part1&Part2.Part3/Part4");
            Assert.AreEqual("Part1&&Part2&.Part3&/Part4", actual);
        }

        /*TODO:
        /// <summary>
        ///A test for FormatErrorMessage
        ///</summary>
        [TestMethod]
        public void FormatErrorMessageTest()
        {
            BrowsePathFormatError formatError = new BrowsePathFormatError(); // TODO: Initialize to an appropriate value
            string expected = string.Empty; // TODO: Initialize to an appropriate value
            string actual;
            actual = BrowsePath.FormatErrorMessage(formatError);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }

        /// <summary>
        ///A test for FromEnumerable
        ///</summary>
        [TestMethod]
        public void FromEnumerableTest()
        {
            IEnumerable<string> enumerable = null; // TODO: Initialize to an appropriate value
            BrowsePath expected = null; // TODO: Initialize to an appropriate value
            BrowsePath actual;
            actual = BrowsePath.FromEnumerable(enumerable);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }
         * */

        /// <summary>
        ///A test for FromString
        ///</summary>
        [TestMethod]
        public void FromStringTest()
        {
            BrowsePath actual = BrowsePath.FromString("/ElementA/ElementB");
            Assert.IsNotNull(actual);
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB" }, actual.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for FromString
        ///</summary>
        [TestMethod]
        public void FromStringTest1()
        {
            BrowsePath actual = BrowsePath.FromString(null);
            Assert.AreEqual(null, actual);
        }

        /// <summary>
        ///A test for GetHashCode - equality
        ///</summary>
        [TestMethod]
        public void GetHashCodeTest1()
        {
            var target1 = new BrowsePath(new[] { "ElementA", "ElementB" });
            var target2 = new BrowsePath(new[] { "ElementA", "ElementB" });
            int expected = target1.GetHashCode();
            int actual = target2.GetHashCode();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for GetHashCode - unequality
        ///</summary>
        [TestMethod]
        public void GetHashCodeTest2()
        {
            var target1 = new BrowsePath(new[] { "ElementA", "ElementB" });
            var target2 = new BrowsePath(new[] { "ElementA", "ElementC" });
            int notExpected = target1.GetHashCode();
            int actual = target2.GetHashCode();
            if (actual == notExpected)
                Assert.Inconclusive();  // not necessarily a failed test
        }

        /// <summary>
        ///A test for IsPrefix
        ///</summary>
        [TestMethod]
        public void IsPrefixTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            var prefix = new BrowsePath(new[] { "ElementA" });
            const bool expected = true;
            bool actual = target.IsPrefix(prefix);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for IsPrefix
        ///</summary>
        [TestMethod]
        public void IsPrefixTest1()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            var prefix = new BrowsePath(new string[0] );
            const bool expected = true;
            bool actual = target.IsPrefix(prefix);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for IsPrefix
        ///</summary>
        [TestMethod]
        public void IsPrefixTest2()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            var prefix = new BrowsePath(new[] { "ElementA", "ElementB" });
            const bool expected = true;
            bool actual = target.IsPrefix(prefix);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for IsPrefix
        ///</summary>
        [TestMethod]
        public void IsPrefixTest3()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            var prefix = new BrowsePath(new[] { "ElementA", "ElementB", "ElementC" });
            const bool expected = false;
            bool actual = target.IsPrefix(prefix);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for IsPrefix
        ///</summary>
        [TestMethod]
        public void IsPrefixTest4()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            var prefix = new BrowsePath(new[] { "ElementC"});
            const bool expected = false;
            bool actual = target.IsPrefix(prefix);
            Assert.AreEqual(expected, actual);
        }

        /*TODO:
        /// <summary>
        ///A test for MakeFormatException
        ///</summary>
        [TestMethod]
        public void MakeFormatExceptionTest()
        {
            BrowsePathFormatError formatError = new BrowsePathFormatError(); // TODO: Initialize to an appropriate value
            FormatException expected = null; // TODO: Initialize to an appropriate value
            FormatException actual;
            actual = BrowsePath.MakeFormatException(formatError);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }
         * */

        /// <summary>
        ///A test for Parse
        ///</summary>
        [TestMethod]
        public void ParseTest()
        {
            var basePath = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string value = "ElementC/ElementD";
            var expected = new BrowsePath(new[] { "ElementA", "ElementB", "ElementC", "ElementD" });
            BrowsePath actual = BrowsePath.Parse(basePath, value);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for Parse
        ///</summary>
        [TestMethod]
        public void ParseTest1()
        {
            var basePath = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string value = "../ElementC/ElementD";
            const int maximumParentDepth = 1;
            var expected = new BrowsePath(new[] { "ElementA", "ElementC", "ElementD" });
            BrowsePath actual = BrowsePath.Parse(basePath, value, maximumParentDepth);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for Parse
        ///</summary>
        [TestMethod]
        public void ParseTest2()
        {
            const string value = "/ElementA/ElementB";
            var expected = new BrowsePath(new[] { "ElementA", "ElementB"});
            BrowsePath actual = BrowsePath.Parse(value);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for RelativeTo
        ///</summary>
        [TestMethod]
        public void RelativeToTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            var toPath = new BrowsePath(new[] { "ElementA", "ElementC", "ElementD" });
            const int maximumParentDepth = 1;
            const string expected = "../ElementC/ElementD";
            string actual = target.RelativeTo(toPath, maximumParentDepth);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for RelativeTo
        ///</summary>
        [TestMethod]
        public void RelativeToTest1()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            var toPath = new BrowsePath(new[] { "ElementA", "ElementB", "ElementC", "ElementD" });
            const string expected = "ElementC/ElementD";
            string actual = target.RelativeTo(toPath);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for ToString
        ///</summary>
        [TestMethod]
        public void ToStringTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            const string expected = "/ElementA/ElementB";
            string actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /*TODO:
        /// <summary>
        ///A test for TryAppendRelative
        ///</summary>
        [TestMethod]
        public void TryAppendRelativeTest()
        {
            BrowsePath browsePath = null; // TODO: Initialize to an appropriate value
            var target = new BrowsePath(browsePath); // TODO: Initialize to an appropriate value
            string more = string.Empty; // TODO: Initialize to an appropriate value
            const BrowsePathFormatError expected = new BrowsePathFormatError(); // TODO: Initialize to an appropriate value
            BrowsePathFormatError actual = target.TryAppendRelative(more);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }

        /// <summary>
        ///A test for TryCombine
        ///</summary>
        [TestMethod]
        public void TryCombineTest()
        {
            BrowsePath browsePath = null; // TODO: Initialize to an appropriate value
            var target = new BrowsePath(browsePath); // TODO: Initialize to an appropriate value
            string more = string.Empty; // TODO: Initialize to an appropriate value
            const BrowsePathFormatError expected = new BrowsePathFormatError(); // TODO: Initialize to an appropriate value
            BrowsePathFormatError actual = target.TryCombine(more);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }
         * */

        /// <summary>
        ///A test for TryParse
        ///</summary>
        [TestMethod]
        public void TryParseTest()
        {
            var basePath = new BrowsePath(new [] {"ElementA", "ElementB"});
            const string value = "ElementC/ElementD";
            const int maximumParentDepth = 0;
            var browsePathExpected = new BrowsePath(new[] { "ElementA", "ElementB", "ElementC", "ElementD" });
            IStringParsingError actual = BrowsePath.TryParse(basePath, value, maximumParentDepth, out BrowsePath browsePath);
            Assert.AreEqual(browsePathExpected, browsePath);
            Assert.IsNull(actual);
        }

        /// <summary>
        ///A test for TryParse
        ///</summary>
        [TestMethod]
        public void TryParseTest1()
        {
            const string value = "/ElementA/ElementB";
            var browsePathExpected = new BrowsePath(new[] { "ElementA", "ElementB" });
            IStringParsingError actual = BrowsePath.TryParse(value, out BrowsePath browsePath);
            Assert.AreEqual(browsePathExpected, browsePath);
            Assert.IsNull(actual);
        }

        /*TODO:
        /// <summary>
        ///A test for TryRelativeTo
        ///</summary>
        [TestMethod]
        public void TryRelativeToTest()
        {
            BrowsePath browsePath = null; // TODO: Initialize to an appropriate value
            BrowsePath target = new BrowsePath(browsePath); // TODO: Initialize to an appropriate value
            BrowsePath toPath = null; // TODO: Initialize to an appropriate value
            const int maximumParentDepth = 0; // TODO: Initialize to an appropriate value
            string result; // TODO: Initialize to an appropriate value
            string resultExpected = string.Empty; // TODO: Initialize to an appropriate value
            const bool expected = false; // TODO: Initialize to an appropriate value
            bool actual = target.TryRelativeTo(toPath, maximumParentDepth, out result);
            Assert.AreEqual(resultExpected, result);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }

        /// <summary>
        ///A test for op_Equality
        ///</summary>
        [TestMethod]
        public void op_EqualityTest()
        {
            BrowsePath left = null; // TODO: Initialize to an appropriate value
            BrowsePath right = null; // TODO: Initialize to an appropriate value
            bool expected = false; // TODO: Initialize to an appropriate value
            bool actual;
            actual = (left == right);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }
         * */

        /// <summary>
        ///A test for op_Implicit
        ///</summary>
        [TestMethod]
        // ReSharper disable InconsistentNaming
#pragma warning disable IDE1006 // Naming Styles
        public void op_ImplicitTest()
#pragma warning restore IDE1006 // Naming Styles
        // ReSharper restore InconsistentNaming
        {
            BrowsePath actual = "/ElementA/ElementB";
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB" }, actual.BrowseNames.ToList());
        }

        /// <summary>
        ///A test for op_Implicit
        ///</summary>
        [TestMethod]
        // ReSharper disable InconsistentNaming
#pragma warning disable IDE1006 // Naming Styles
        public void op_ImplicitTest1()
#pragma warning restore IDE1006 // Naming Styles
        // ReSharper restore InconsistentNaming
        {
            BrowsePath actual = (string)null;
            // ReSharper disable ExpressionIsAlwaysNull
            Assert.AreEqual(null, actual);
            // ReSharper restore ExpressionIsAlwaysNull
        }

        /*TODO:
        /// <summary>
        ///A test for op_Inequality
        ///</summary>
        [TestMethod]
        public void op_InequalityTest()
        {
            BrowsePath left = null; // TODO: Initialize to an appropriate value
            BrowsePath right = null; // TODO: Initialize to an appropriate value
            bool expected = false; // TODO: Initialize to an appropriate value
            bool actual;
            actual = (left != right);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }
         * */

        /// <summary>
        ///A test for BrowseNames
        ///</summary>
        [TestMethod]
        public void BrowseNamesTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            IEnumerable<string> actual = target.BrowseNames;
            CollectionAssert.AreEqual(new[] { "ElementA", "ElementB" }, actual.ToList());
        }

        /// <summary>
        ///A test for IsRoot
        ///</summary>
        [TestMethod]
        public void IsRootTest()
        {
            var target = new BrowsePath(new string[0]);
            bool actual = target.IsEmpty;
            Assert.AreEqual(true, actual);
        }

        /// <summary>
        ///A test for IsRoot
        ///</summary>
        [TestMethod]
        public void IsRootTest1()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            bool actual = target.IsEmpty;
            Assert.AreEqual(false, actual);
        }

        /// <summary>
        ///A test for Length
        ///</summary>
        [TestMethod]
        public void LengthTest()
        {
            var target = new BrowsePath(new[] { "ElementA", "ElementB" });
            int actual = target.Length;
            Assert.AreEqual(2, actual);
        }

        /// <summary>
        ///A test for PathString
        ///</summary>
        [TestMethod]
        public void PathStringTest()
        {
            var target = new BrowsePath();
            const string expected = "/ElementA/ElementB";
            target.PathString = expected;
            string actual = target.PathString;
            Assert.AreEqual(expected, actual);
        }

        /*TODO:
        /// <summary>
        ///A test for TryAssign
        ///</summary>
        [TestMethod]
        public void TryAssignTest()
        {
            BrowsePath target = new BrowsePath(); // TODO: Initialize to an appropriate value
            string absolute = string.Empty; // TODO: Initialize to an appropriate value
            BrowsePathFormatError expected = new BrowsePathFormatError(); // TODO: Initialize to an appropriate value
            BrowsePathFormatError actual;
            actual = target.TryAssign(absolute);
            Assert.AreEqual(expected, actual);
            Assert.Inconclusive("Verify the correctness of this test method.");
        }
         * */

        [TestMethod]
        // ReSharper disable InconsistentNaming
        public void ParseTest_DiggerFound()
        // ReSharper restore InconsistentNaming
        {
            try
            {
                // ReSharper disable ReturnValueOfPureMethodIsNotUsed
                BrowsePath.Parse("/.\0");
                // ReSharper restore ReturnValueOfPureMethodIsNotUsed
            }
            catch (BrowsePathFormatException browsePathFormatException)
            {
                StringAssert.Contains(browsePathFormatException.Message, "reference");
                StringAssert.Contains(browsePathFormatException.Message, "invalid");
                return;
            }
            Assert.Fail();
        }
    }
}
