﻿using System;
using System.Threading;
using System.Windows.Forms;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib;
using OpcLabs.BaseLib.ComponentModel;
using OpcLabs.BaseLib.Forms.Common;
using OpcLabs.BaseLib.Forms.Common.Utilities;
using OpcLabs.BaseLib.Forms.IdentityModel;
using OpcLabs.BaseLib.IdentityModel.User;
using OpcLabs.BaseLib.OperationModel.Generic;
using OpcLabs.BaseLib.Threading;
using OpcLabs.DotProlog.Code;
using OpcLabs.DotProlog.Terms;
using OpcLabs.DotProlog.Test;

namespace OpcLabs.BaseLibTest.Forms.Common.Utilities
{
    [TestClass]
    public class ValueControlUtilitiesTest
    {
        [TestClass]
        // ReSharper disable once InconsistentNaming
        public class DerivedType_2_Test : PredicateTest
        {
            public DerivedType_2_Test() : base("derived_type", 2)
            {
            }

            [TestMethod]
            public void Test1()
            {
                Exception exception = null;
                void ThreadAction()
                {
                    try
                    {
                        var assignableClrType = new Variable();
                        TestPredicate(
                            new Term[]
                            {
                                new ClrObject(typeof(TokenInfo)),
                                assignableClrType
                            },
                            new[]
                            {
                                new VariableDictionary {{assignableClrType, new ClrObject(typeof(AnonymousTokenInfo))}},
                                new VariableDictionary {{assignableClrType, new ClrObject(typeof(KerberosTokenInfo))}},
                                new VariableDictionary {{assignableClrType, new ClrObject(typeof(UserNameTokenInfo))}},
                                new VariableDictionary {{assignableClrType, new ClrObject(typeof(X509CertificateTokenInfo))}}
                            });
                    }
                    catch (Exception e)
                    {
                        exception = e;
                    }
                }

                var thread = new Thread(ThreadAction, 10 * 1024 * 1024);
                thread.Start();
                thread.Join();
                if (!(exception is null))
                    throw exception;
            }

            //protected override IDebugger CreateDebugger() => new NullDebugger();    // the amount of tracing output is too big

            protected override IProgram CreateProgram()
            {
                var module = new Module();
                // ReSharper disable once AssignNullToNotNullAttribute
                ValueControlUtilities.ConsultAll(module.Database, Parallelism.Default);
                return new Program(module);
            }
        }

        [TestClass]
        // ReSharper disable once InconsistentNaming
        public class SatisfyingGroundSemiTerminalValueControl_7_Test : PredicateTest
        {
            public SatisfyingGroundSemiTerminalValueControl_7_Test() : base("satisfying_ground_semi_terminal_value_control", 7)
            {
            }

            [TestMethod]
            public void Test1()
            {
                var pageId = new Variable();
                var nullable = new Variable();
                var polymorphic = new Variable();
                var readWrite = new Variable();
                var control = new Variable();
                TestPredicate(
                    new Term[]
                    {
                        /*ClrEditType*/ new ClrObject(typeof(UserIdentity)),
                        pageId,
                        nullable,
                        polymorphic,
                        readWrite,
                        /*ExportedOnly*/ Atom.Add("false"),
                        control
                    },
                    new []
                    {
                        new VariableDictionary
                        {
                            {pageId, new PInteger(ControlPageIds.Properties)},
                            {nullable, Atom.Add("false")},
                            {polymorphic, Atom.Add("false")},
                            {readWrite, Atom.Add("true")},
                            {control, new NTerm("object",
                                new ClrObject(typeof(PropertyGridControl)),
                                TermAtoms.EmptyList,
                                TermAtoms.EmptyList) },
                        },
                        new VariableDictionary
                        {
                            {pageId, new PInteger(ControlPageIds.Properties)},
                            {nullable, Atom.Add("false")},
                            {polymorphic, Atom.Add("false")},
                            {readWrite, Atom.Add("false")},
                            {control, new NTerm("object")
                            {
                                new ClrObject(typeof(PropertyGridControl)), new NList(), new NList
                                {
                                    new NTerm("="){"RequestReadWrite", new NTerm("boolean"){"false"}}
                                }
                            }}
                        },
                        new VariableDictionary
                        {
                            {pageId, new PInteger(ControlPageIds.ViewOrEdit)},
                            {nullable, Atom.Add("false")},
                            {polymorphic, Atom.Add("false")},
                            {readWrite, Atom.Add("true")},
                            {control, new NTerm("object",
                                new ClrObject(typeof(UserIdentityControl)),
                                TermAtoms.EmptyList,
                                TermAtoms.EmptyList) },
                        },
                    });
            }

            protected override IProgram CreateProgram()
            {
                var module = new Module();
                // ReSharper disable once AssignNullToNotNullAttribute
                ValueControlUtilities.ConsultAll(module.Database, Parallelism.Default);
                return new Program(module);
            }
        }

        [TestClass]
        // ReSharper disable once InconsistentNaming
        public class SatisfyingGroundValueControl_7_Test : PredicateTest
        {
            public SatisfyingGroundValueControl_7_Test() : base("satisfying_ground_value_control", 7)
            {
            }

            [TestMethod]
            public void Test1()
            {
                var id = new Variable();
                var nullable = new Variable();
                var polymorphic = new Variable();
                Atom readWrite = Atom.Add("true");
                var control = new Variable();
                TestPredicate(
                    new Term[]
                    {
                        /*ClrEditType*/ new ClrObject(typeof(UserIdentity)),
                        id,
                        nullable,
                        polymorphic,
                        readWrite,
                        /*ExportedOnly*/ Atom.Add("false"),
                        control
                    },
                    new[]
                    {
                        new VariableDictionary
                        {
                            {id, new PInteger(ControlPageIds.Multi)},
                            {nullable, Atom.Add("false")},
                            {polymorphic, Atom.Add("false")},
                            {control, new NTerm("object"){ new ClrObject(typeof(ObjectEditorControl)), new NList(), new NList
                            {
                                new NTerm("="){"InnerControls", new NList
                                {
                                    new NList
                                    {
                                        new NTerm("object"){new ClrObject(typeof(PropertyGridControl)), new NList(), new NList()},
                                        new NTerm("object"){new ClrObject(typeof(UserIdentityControl)), new NList(), new NList()}
                                    }
                                }}
                            }}}
                        },
                        new VariableDictionary
                        {
                            {id, new PInteger(ControlPageIds.Multi)},
                            {nullable, Atom.Add("false")},
                            {polymorphic, Atom.Add("false")},
                            {control, new NTerm("object"){ new ClrObject(typeof(ObjectEditorControl)), new NList(), new NList
                            {
                                new NTerm("="){"InnerControls", new NList
                                {
                                    new NList
                                    {
                                        new NTerm("object"){new ClrObject(typeof(PropertyGridControl)), new NList(), new NList
                                        {
                                            new NTerm("="){"RequestReadWrite", new NTerm("boolean"){"false"}}
                                        }},
                                        new NTerm("object"){new ClrObject(typeof(UserIdentityControl)), new NList(), new NList()}
                                    }
                                }}
                            }}}
                        },
                        new VariableDictionary
                        {
                            {id, new PInteger(ControlPageIds.Multi)},
                            {nullable, Atom.Add("true")},
                            {polymorphic, Atom.Add("false")},
                            {control, new NTerm("object"){new ClrObject(typeof(NullableValueControl)), new NList(), new NList
                                {
                                    new NTerm("="){"EditType", new ClrObject(typeof(UserIdentity))},
                                    new NTerm("="){"InnerControl", new NTerm("object"){ new ClrObject(typeof(ObjectEditorControl)), new NList(), new NList
                                        {
                                            new NTerm("="){"InnerControls", new NList
                                            {
                                                new NList
                                                {
                                                    new NTerm("object"){new ClrObject(typeof(PropertyGridControl)), new NList(), new NList()},
                                                    new NTerm("object"){new ClrObject(typeof(UserIdentityControl)), new NList(), new NList()}
                                                }
                                            }}
                                        }}}
                                }
                            }}
                        }
                    },
                    maximumResultCount:3    // TODO: test more results
                );
            }

            [TestMethod]
            public void Test2()
            {
                var id = new Variable();
                var nullable = new Variable();
                var polymorphic = new Variable();
                var readWrite = new Variable();
                var control = new Variable();
                TestPredicate(
                    new Term[]
                    {
                        /*ClrEditType*/ new ClrObject(typeof(Info)),
                        id,
                        nullable,
                        polymorphic,
                        readWrite,
                        /*ExportedOnly*/ Atom.Add("false"),
                        control
                    },
                    new[]
                    {
                        // TODO: why don't these come out, first?
                        //new VariableDictionary
                        //{
                        //    {id, new PInteger(ControlPageIds.Properties)},
                        //    {nullable, Atom.Add("false")},
                        //    {polymorphic, Atom.Add("false")},
                        //    {readWrite, Atom.Add("true")},
                        //    {control, new NTerm("object"){ new ClrObject(typeof(PropertyGridControl)), new NList(), new NList() }}
                        //},
                        //new VariableDictionary
                        //{
                        //    {id, new PInteger(ControlPageIds.Properties)},
                        //    {nullable, Atom.Add("false")},
                        //    {polymorphic, Atom.Add("false")},
                        //    {readWrite, Atom.Add("false")},
                        //    {control, new NTerm("object"){ new ClrObject(typeof(PropertyGridControl)), new NList(), new NList
                        //    {
                        //        new NTerm("="){"RequestReadWrite", new NTerm("boolean"){"false"}}
                        //    }}}
                        //}
                        new VariableDictionary
                        {
                            {id, new PInteger(ControlPageIds.Multi)},
                            {nullable, Atom.Add("false")},
                            {polymorphic, Atom.Add("false")},
                            {readWrite, Atom.Add("true")},
                            {control, new NTerm("object"){ new ClrObject(typeof(ObjectEditorControl)), new NList(), new NList
                            {
                                new NTerm("="){"InnerControls", new NList
                                {
                                    new NList
                                    {
                                        new NTerm("object"){new ClrObject(typeof(PropertyGridControl)), new NList(), new NList()},
                                    }
                                }}
                            }}}
                        },
                        new VariableDictionary
                        {
                        {id, new PInteger(ControlPageIds.Multi)},
                        {nullable, Atom.Add("false")},
                        {polymorphic, Atom.Add("false")},
                        {readWrite, Atom.Add("false")},
                        {control, new NTerm("object"){ new ClrObject(typeof(ObjectEditorControl)), new NList(), new NList
                        {
                            new NTerm("="){"InnerControls", new NList
                            {
                                new NList
                                {
                                    new NTerm("object"){new ClrObject(typeof(PropertyGridControl)), new NList(), new NList{
                                        new NTerm("="){"RequestReadWrite", new NTerm("boolean"){"false"}}}}
                                }
                            }}
                        }}}
                        },
                    },
                    maximumResultCount: 2    // TODO: test more results
                );
            }

            protected override IProgram CreateProgram()
            {
                var module = new Module();
                // ReSharper disable once AssignNullToNotNullAttribute
                ValueControlUtilities.ConsultAll(module.Database, Parallelism.Default);
                return new Program(module);
            }
        }

        [TestClass]
        // ReSharper disable once InconsistentNaming
        public class TerminalValueControl_11_Test : PredicateTest
        {
            public TerminalValueControl_11_Test() : base("terminal_value_control", 11)
            {
            }

            [TestMethod]
            public void Test1()
            {
                var pageId = new Variable();
                var canChangeNullable = new Variable();
                var canChangePolymorphic = new Variable();
                var canChangeReadWrite = new Variable();
                var defaultNullable = new Variable();
                var defaultPolymorphic = new Variable();
                var defaultReadWrite = new Variable();
                var control = new Variable();
                TestPredicate(
                    new Term[]
                    {
                        /*ClrEditType*/ new ClrObject(typeof(UserIdentity)),
                        /*ElementPath*/ new PString(""),
                        pageId,
                        canChangeNullable,
                        canChangePolymorphic,
                        canChangeReadWrite,
                        defaultNullable,
                        defaultPolymorphic,
                        defaultReadWrite,
                        /*ExportedOnly*/ Atom.Add("false"),
                        control
                    },
                    new []
                    {
                        new VariableDictionary
                        {
                            {control, new NTerm("object", 
                                new ClrObject(typeof(PropertyGridControl)), 
                                TermAtoms.EmptyList, 
                                TermAtoms.EmptyList) },
                            {pageId, new PInteger(1000) }
                        },
                        new VariableDictionary
                        {
                            {control, new NTerm("object",
                                new ClrObject(typeof(PropertyGridControl)), 
                                TermAtoms.EmptyList, 
                                new NList {new NTerm("=", Atom.Add("RequestReadWrite"), new Variable())}) },
                            {pageId, new PInteger(1000) }
                        },
                        new VariableDictionary
                        {
                            {control, new NTerm("object", 
                                new ClrObject(typeof(UserIdentityControl)), 
                                TermAtoms.EmptyList, 
                                TermAtoms.EmptyList) },
                            {pageId, new PInteger(0) }
                        }
                    },
                    maximumResultCount:1);  // TODO: test more!
            }

            protected override IProgram CreateProgram()
            {
                var module = new Module();
                // ReSharper disable once AssignNullToNotNullAttribute
                ValueControlUtilities.ConsultAll(module.Database, Parallelism.Default);
                return new Program(module);
            }
        }

        [TestMethod()]
        public void FirstSatisfyingValueClrControlTest_EditOrView()
        {
            ValueResult<Control> valueResult = ValueControlUtilities.FirstSatisfyingValueClrControl<Control>(
                typeof(UserIdentity),
                ControlPageIds.ViewOrEdit,
                nullable: false,
                polymorphic: false,
                readWrite: true,
                exportedOnly:false,
                getType:null,
                getTypes:null,
                designMode:false);
            Assert.IsNotNull(valueResult);
            Assert.IsTrue(valueResult.Succeeded);
            var @object = valueResult.Value;
            Assert.IsNotNull(@object);
            Assert.IsInstanceOfType(@object, typeof(UserIdentityControl));
        }

        [TestMethod()]
        public void FirstSatisfyingValueClrControlTest_EditOrView_Nullable()
        {
            ValueResult<Control> valueResult = ValueControlUtilities.FirstSatisfyingValueClrControl<Control>(
                typeof(UserIdentity),
                ControlPageIds.ViewOrEdit,
                nullable: true,
                polymorphic: false,
                readWrite: true,
                exportedOnly: false,
                getType: null,
                getTypes:null,
                designMode: false);
            Assert.IsNotNull(valueResult);
            Assert.IsTrue(valueResult.Succeeded);
            var @object = valueResult.Value;
            Assert.IsNotNull(@object);
            Assert.IsInstanceOfType(@object, typeof(NullableValueControl));
            var nullableValueControl = (NullableValueControl) @object;
            Assert.IsNotNull(nullableValueControl.InnerControl);
            Assert.IsInstanceOfType(nullableValueControl.InnerControl, typeof(UserIdentityControl));
            Assert.AreEqual(typeof(UserIdentity), nullableValueControl.EditType);
        }

        [TestMethod()]
        public void FirstSatisfyingValueClrControlTest_Properties()
        {
            ValueResult<Control> valueResult = ValueControlUtilities.FirstSatisfyingValueClrControl<Control>(
                typeof(UserIdentity),
                ControlPageIds.Properties,
                nullable: false,
                polymorphic: false,
                readWrite: true,
                exportedOnly: false,
                getType: null,
                getTypes:null,
                designMode: false);
            Assert.IsNotNull(valueResult);
            Assert.IsTrue(valueResult.Succeeded);
            var @object = valueResult.Value;
            Assert.IsNotNull(@object);
            Assert.IsInstanceOfType(@object, typeof(PropertyGridControl));
        }

        [TestMethod()]
        public void FirstSatisfyingValueClrControlTest_Object()
        {
            ValueResult<Control> valueResult = ValueControlUtilities.FirstSatisfyingValueClrControl<Control>(
                typeof(UserIdentity),
                ControlPageIds.Multi,
                nullable: false,
                polymorphic: false,
                readWrite: true,
                exportedOnly: false,
                getType: null,
                getTypes:null,
                designMode: false);
            Assert.IsNotNull(valueResult);
            Assert.IsTrue(valueResult.Succeeded);
            var @object = valueResult.Value;
            Assert.IsNotNull(@object);
            Assert.IsInstanceOfType(@object, typeof(ObjectEditorControl));

            // TODO: test InnerControls
        }
    }
}
