﻿
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.DataTypeModel;
using OpcLabs.BaseLib.Extensions;
using OpcLabs.BaseLib.OperationModel.Generic;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.EasyOpc.UA.DataTypeModel.Extensions;
using OpcLabs.EasyOpc.UA.InformationModel;
using OpcLabs.EasyOpc.UA.Plugins.ComplexData;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA.Plugins.ComplexData
{
    [TestClass]
    // ReSharper disable once InconsistentNaming
    public class IEasyUAClientComplexDataTest
    {
        #region Test Methods

        [TestMethod]
        public void ResolveDataTypeTest_FromDataTypeId_OpcFoundation_SampleServer_ArrayValueDataType() => ResolveDataTypeTest_FromDataTypeId(
            UALiveTestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp,
            OpcFoundation_UASampleServer.NodeIdExpandedTexts.DataType_ArrayValueDataType);

        [TestMethod]
        public void ResolveDataTypeTest_FromDataTypeId_OpcFoundation_SampleServer_ScalarValueDataType() => ResolveDataTypeTest_FromDataTypeId(
            UALiveTestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp,
            OpcFoundation_UASampleServer.NodeIdExpandedTexts.DataType_ScalarValueDataType);

        //[TestMethod]
        //public void ResolveDataTypeTest_FromModelNode_OpcFoundation_SampleServer() => ResolveDataTypeTest(
        //    TestEnvironment.OpcFoundation_SampleServer.EndpointDescriptors.OpcTcp,
        //    OpcFoundation_SampleServer.NodeIdExpandedTexts.Data_Static_Scalar_StructureValue);

        [TestMethod]
        public void ResolveDataTypeTest_FromModelNode_Softing_OpcUaNetDemoServer_DefaultBinary() => ResolveDataTypeTest_FromModelNode(
            UALiveTestEnvironment.Softing_OpcUaNetDemoServer.EndpointDescriptors.OpcTcp,
            "nsu=http://industrial.softing.com/UA/Refrigerator;ns=10;i=13");

        #endregion



        #region Private Properties

        [NotNull]
        private UALiveTestEnvironment TestEnvironment => UALiveTestEnvironment.Current;

        #endregion



        #region Private Methods

        private void ResolveDataTypeTest_FromDataTypeId(
            [NotNull] UAEndpointDescriptor endpointDescriptor,
            [NotNull] UANodeDescriptor dataTypeId)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            EasyUAClient client = new EasyUAClient();
            IEasyUAClientComplexData clientComplexData = client.GetService<IEasyUAClientComplexData>();
            Assert.IsNotNull(clientComplexData, nameof(clientComplexData));

            TestSection.Act();

            ValueResult<DataType> dataTypeResult = clientComplexData.ResolveDataType(
                new UAModelNodeDescriptor(endpointDescriptor, dataTypeId),
                UABrowseNames.DefaultBinary);

            TestSection.Assert();

            AssertUtilities.WriteLine($"dataTypeResult: {dataTypeResult}");
            Assert.IsNotNull(dataTypeResult);
            Assert.IsTrue(dataTypeResult.Succeeded);
            DataType dataType = dataTypeResult.Value;
            AssertUtilities.WriteLine($"dataType: {dataType}");
            Assert.IsNotNull(dataType);
            AssertUtilities.WriteLine($"{dataType:V}");
        }

        private void ResolveDataTypeTest_FromModelNode(
            [NotNull] UAEndpointDescriptor endpointDescriptor,
            [NotNull] UANodeDescriptor nodeDescriptor)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            EasyUAClient client = new EasyUAClient();
            IEasyUAClientComplexData clientComplexData = client.GetService<IEasyUAClientComplexData>();
            Assert.IsNotNull(clientComplexData, nameof(clientComplexData));

            ValueResult<UAModelNodeDescriptor> dataTypeIdResult =
                clientComplexData.DataTypeModelProvider.GetDataTypeIdFromModelNodeDescriptor(
                    new UAModelNodeDescriptor(endpointDescriptor, nodeDescriptor));
            AssertUtilities.WriteLine($"dataTypeIdResult: {dataTypeIdResult}");
            Assert.IsNotNull(dataTypeIdResult);
            Assert.IsTrue(dataTypeIdResult.Succeeded);
            Assert.IsNotNull(dataTypeIdResult.Value);
            UANodeDescriptor dataTypeId = dataTypeIdResult.Value.NodeDescriptor;
            //UANodeId dataTypeId = client.ReadDataType(endpointDescriptor, nodeDescriptor);

            AssertUtilities.WriteLine($"dataTypeId: {dataTypeId}");
            Assert.IsNotNull(dataTypeId);

            TestSection.Act();

            ValueResult<DataType> dataTypeResult = clientComplexData.ResolveDataType(
                new UAModelNodeDescriptor(endpointDescriptor, dataTypeId),
                UABrowseNames.DefaultBinary);

            TestSection.Assert();

            AssertUtilities.WriteLine($"dataTypeResult: {dataTypeResult}");
            Assert.IsNotNull(dataTypeResult);
            Assert.IsTrue(dataTypeResult.Succeeded);
            DataType dataType = dataTypeResult.Value;
            AssertUtilities.WriteLine($"dataType: {dataType}");
            Assert.IsNotNull(dataType);
            AssertUtilities.WriteLine($"{dataType:V}");
        }

        #endregion
    }
}
