﻿
using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.Generic;
using OpcLabs.EasyOpc.UA.Reactive;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA.Reactive
{
    [TestClass]
    public class UADataChangeNotificationObservableTest
    {
        #region Private Types

        private class TestObserver : IObserver<EasyUADataChangeNotificationEventArgs<int>>
        {
            public void OnNext(EasyUADataChangeNotificationEventArgs<int> eventArgs)
            {
                // IMPROVE: Asserts or exceptions from inside non-test threads don't actually work
                Assert.IsNotNull(eventArgs);
                Assert.IsNotNull(eventArgs.TypedAttributeData);
                AssertUtilities.WriteLine(eventArgs.TypedAttributeData);
            }

            public void OnError(Exception error)
            {
                Assert.Fail("OnError called");
            }

            public void OnCompleted()
            {
                AssertUtilities.WriteLine("OnCompleted called");
            }
        }

        #endregion



        #region Test Methods

        // IMPROVE: this test does not differ much from UAReactiveTest.MonitoredItemChangedObservableTest
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void SubscribeTest()
        {
            TestSection.Arrange();
            UAEndpointDescriptor endpointDescriptor = UALiveTestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            UADataChangeNotificationObservable<int> target = UADataChangeNotificationObservable.Create<int>(
                endpointDescriptor,
                OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Dynamic_UserScalar_Int32Value,
                100);

            TestSection.Act();

            using (target.Subscribe(new TestObserver()))
                AssertUtilities.WaitingPhase(20 * 1000);

            TestSection.Assert();
        }

        #endregion



        #region Private Properties

        [NotNull]
        private UALiveTestEnvironment TestEnvironment => UALiveTestEnvironment.Current;

        #endregion
    }
}
