﻿
using System;
using System.Linq;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.IdentityModel.User;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.UnitTestLib;

namespace OpcLabs.OpcInteroperabilityTest.UA
{
    partial class UAInteroperabilityWorkshopTest
    {
        #region Static Private Methods

        //static private void TestDataTypesFolder([NotNull] EasyUAClient client, [NotNull] UAEndpointDescriptor endpointDescriptor)
        //{
        //    UANodeElementCollection nodeElementCollection = client.Browse(endpointDescriptor, UAObjectIds.DataTypesFolder,
        //        UABrowseParameters.AllForwardReferences);
        //    foreach (UANodeElement nodeElement in nodeElementCollection)
        //        // ReSharper disable once PossibleNullReferenceException
        //        AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
        //    AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");

        //    // ReSharper disable PossibleNullReferenceException

        //    // ReSharper restore PossibleNullReferenceException
        //}

        static private void TestObjectTypesFolder([NotNull] EasyUAClient client, [NotNull] UAEndpointDescriptor endpointDescriptor)
        {
            UANodeElementCollection nodeElementCollection = client.Browse(endpointDescriptor, UAObjectIds.ObjectTypesFolder,
                UABrowseParameters.AllForwardReferences);
            foreach (UANodeElement nodeElement in nodeElementCollection)
                // ReSharper disable once PossibleNullReferenceException
                AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
            AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");

            // ReSharper disable PossibleNullReferenceException
            Assert.IsTrue(nodeElementCollection.Any(element => element.NodeId.Match(UAObjectTypeIds.BaseObjectType)));
            Assert.IsTrue(nodeElementCollection.Any(element => element.NodeId.Match(UAObjectTypeIds.FolderType)));
            // ReSharper restore PossibleNullReferenceException
        }

        static private void TestThatSomeNodesArePresent(
            [NotNull] EasyUAClient client,
            [NotNull] UAEndpointDescriptor endpointDescriptor,
            [NotNull] UANodeDescriptor parentNodeDescriptor)
        {
            AssertUtilities.WriteLine($"Parent node: {parentNodeDescriptor}");
            UANodeElementCollection nodeElementCollection = client.Browse(endpointDescriptor, parentNodeDescriptor, UABrowseParameters.AllForwardReferences);
            foreach (UANodeElement nodeElement in nodeElementCollection)
                // ReSharper disable once PossibleNullReferenceException
                AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
            AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");
            Assert.IsTrue(nodeElementCollection.Count >= 1, "Not enough elements");
        }

        static private void TestTypesFolder([NotNull] EasyUAClient client, [NotNull] UAEndpointDescriptor endpointDescriptor)
        {
            UANodeElementCollection nodeElementCollection = client.Browse(endpointDescriptor, UAObjectIds.TypesFolder,
                UABrowseParameters.AllForwardReferences);
            foreach (UANodeElement nodeElement in nodeElementCollection)
                // ReSharper disable once PossibleNullReferenceException
                AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
            AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");

            // ReSharper disable PossibleNullReferenceException
            Assert.IsTrue(nodeElementCollection.Any(element => element.NodeId.Match(UAObjectIds.ReferenceTypesFolder)));
            Assert.IsTrue(nodeElementCollection.Any(element => element.NodeId.Match(UAObjectIds.DataTypesFolder)));
            // ReSharper restore PossibleNullReferenceException
        }

        #endregion



        #region Test Methods

        [TestMethod]
        public void Group005_InformationModel_Header____________________()
        {
        }

        /// <summary>
        /// Browse() for standard folders: Objects, Types, ReferenceTypes, ObjectTypes, VariableTypes, DataTypes, and Views.
        /// </summary>
        [TestMethod]
        public void Group005_InformationModel_Test01_Browse_StandardFolders()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;

            TestSection.Act();
            using (var client = new EasyUAClient { Isolated = true })
            {
                TestThatSomeNodesArePresent(client, endpointDescriptor, UAObjectIds.ObjectsFolder);
                TestThatSomeNodesArePresent(client, endpointDescriptor, UAObjectIds.TypesFolder);
                TestThatSomeNodesArePresent(client, endpointDescriptor, UAObjectIds.ReferenceTypesFolder);
                TestThatSomeNodesArePresent(client, endpointDescriptor, UAObjectIds.ObjectTypesFolder);
                TestThatSomeNodesArePresent(client, endpointDescriptor, UAObjectIds.VariableTypesFolder);
                TestThatSomeNodesArePresent(client, endpointDescriptor, UAObjectIds.DataTypesFolder);
                TestThatSomeNodesArePresent(client, endpointDescriptor, UAObjectIds.ViewsFolder);
            }
        }

        /// <summary>
        /// Browse() the Types folder and verify standard types are exposed, e.g. BaseObjectType, FolderType, Reference Types, Data Types (scalar types and standard structures).
        /// </summary>
        [TestMethod]
        public void Group005_InformationModel_Test02_Browse_TypesFolder()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;

            TestSection.Act();
            using (var client = new EasyUAClient { Isolated = true })
            {
                TestObjectTypesFolder(client, endpointDescriptor);
                TestTypesFolder(client, endpointDescriptor);
                //TestDataTypesFolder(client, endpointDescriptor);  // IMPROVE: have this
            }
        }

        /// <summary>
        /// Browse() the Server Object for ServerCapabilities, ServerDiagnostics, ServerStatus, VendorServerInfo, and any other standardized components.
        /// </summary>
        [TestMethod]
        public void Group005_InformationModel_Test03_Browse_ServerFolder()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;

            TestSection.Act();
            using (var client = new EasyUAClient { Isolated = true })
            {
                UANodeElementCollection nodeElementCollection = client.BrowseDataNodes(endpointDescriptor, UAObjectIds.Server);
                foreach (UANodeElement nodeElement in nodeElementCollection)
                    // ReSharper disable once PossibleNullReferenceException
                    AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
                AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");

                // ReSharper disable PossibleNullReferenceException
                Assert.IsTrue(nodeElementCollection.Any(element => element.NodeId.Match(UAObjectIds.Server_ServerCapabilities)));
                Assert.IsTrue(nodeElementCollection.Any(element => element.NodeId.Match(UAObjectIds.Server_ServerDiagnostics)));
                Assert.IsTrue(nodeElementCollection.Any(element => element.NodeId.Match(UAVariableIds.Server_ServerStatus)));
                Assert.IsTrue(nodeElementCollection.Any(element => element.NodeId.Match(UAObjectIds.Server_VendorServerInfo)));
                // IMPROVE: have more
                // ReSharper restore PossibleNullReferenceException
            }
        }

        /// <summary>
        /// Browse() the ServerDiagnostics.SessionDiagnosticsArray and verify the Client connection and its details can be read
        /// </summary>
        [TestMethod]
        public void Group005_InformationModel_Test04_Browse_Read_SessionDiagnosticsArray()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;
            if (UserName != null)
                endpointDescriptor = new UAEndpointDescriptor(endpointDescriptor)
                { UserIdentity = UserIdentity.CreateUserNameIdentity(UserName, Password) };

            UANodeDescriptor sessionDiagnosticsNodeDescriptor =
                //UAObjectIds.Server_ServerDiagnostics_SessionsDiagnosticsSummary;
                UAVariableIds.Server_ServerDiagnostics_SessionsDiagnosticsSummary_SessionDiagnosticsArray;
            AssertUtilities.WriteLine($"Node descriptor 1: {sessionDiagnosticsNodeDescriptor}");
            AssertUtilities.WriteLine($"...expanded text: {sessionDiagnosticsNodeDescriptor.NodeId.ExpandedText}");

            TestSection.Act();
            using (var client = new EasyUAClient {Isolated = true})
            {
                UANodeElementCollection nodeElementCollection1 =
                    client.BrowseDataNodes(endpointDescriptor, sessionDiagnosticsNodeDescriptor);
                foreach (UANodeElement nodeElement in nodeElementCollection1)
                    // ReSharper disable once PossibleNullReferenceException
                    AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
                AssertUtilities.WriteLine($"Element count: {nodeElementCollection1.Count}");

                AssertUtilities.WriteLine();
                UANodeDescriptor sessionDiagnosticsArrayNodeDescriptor = UAVariableIds
                    .Server_ServerDiagnostics_SessionsDiagnosticsSummary_SessionDiagnosticsArray;
                AssertUtilities.WriteLine($"Node descriptor 2: {sessionDiagnosticsArrayNodeDescriptor}");
                AssertUtilities.WriteLine(
                    $"...expanded text: {sessionDiagnosticsArrayNodeDescriptor.NodeId.ExpandedText}");
                object value = client.ReadValue(endpointDescriptor, sessionDiagnosticsArrayNodeDescriptor);
                AssertUtilities.WriteLine($"Value: {value}");

                TestSection.Assert();

                Assert.IsNotNull(value);
                Type type = value.GetType();
                AssertUtilities.WriteLine($"Type: {type}");
                Assert.IsInstanceOfType(value, typeof(object[]));
                var sessionDiagnosticsArray = (object[]) value;
                int count = sessionDiagnosticsArray.Length;
                AssertUtilities.WriteLine($"Count: {count}");
                for (int i = 0; i < count; i++)
                {
                    object element = sessionDiagnosticsArray[i];
                    AssertUtilities.WriteLine($"Element {i}: {element}");
                    Assert.IsNotNull(element);
                    string typeName = element.GetType().FullName;
                    Assert.AreEqual("Opc.Ua.SessionDiagnosticsDataType", typeName);
                }

                Assert.IsTrue(count >= 1, "At least one session expected");
            }
        }

        #endregion
    }
}
